@@ -4,7 +4,7 @@ concatenate_colnames <- function(var_name, prefix, include_mean = TRUE) {
     lapply(
       prefix,
       function(lag) paste(var_name, lag, sep = "_")
-    ) %>% 
+    ) |> 
     unlist()
   if (!include_mean) {
     return(nm)
@@ -33,11 +33,11 @@ split_coef <- function(object, ...) {
     return(
       switch(object$type,
         "const" = {
-          split.data.frame(object$coefficients[-object$df, ], gl(object$p, object$m)) %>%
+          split.data.frame(object$coefficients[-object$df, ], gl(object$p, object$m)) |>
             lapply(t)
         },
         "none" = {
-          split.data.frame(object$coefficients, gl(object$p, object$m)) %>%
+          split.data.frame(object$coefficients, gl(object$p, object$m)) |>
             lapply(t)
         }
       )
@@ -53,62 +53,6 @@ split_coef <- function(object, ...) {
   }
 }
 
-#' Changing 3d initial array Input to List
-#' 
-#' This function changes 3d array of [init_ssvs()] function to list.
-#' 
-#' @param init_array potentially 3d array initial input
-#' 
-#' @noRd
-change_to_list <- function(init_array) {
-  if (length(dim(init_array)) == 3) {
-    lapply(
-      seq_len(dim(init_array)[3]),
-      function(k) init_array[,, k]
-    )
-  } else {
-    init_array
-  }
-}
-
-#' Checking if the Parallel Initial List are not Identical
-#' 
-#' This function checks if the list of parallel initial matrices are identical.
-#' 
-#' @param init_list List of parallel initial matrix
-#' @param case Check dimension (`dim`) or values (`values`).
-#' 
-#' @noRd
-isnot_identical <- function(init_list, case = c("dim", "values")) {
-  case <- match.arg(case)
-  switch(
-    case,
-    "dim" = {
-      if (length(unique(lapply(init_list, dim))) != 1) {
-        stop(paste0(
-          "Dimension of '",
-          deparse(substitute(init_list)),
-          "' across every chain should be the same."
-        ))
-      }
-    },
-    "values" = {
-      if (any(unlist(
-        lapply(
-          seq_along(init_list)[-1], 
-          function(i) identical(init_list[[1]], init_list[[i]])
-        )
-      ))) {
-        warning(paste0(
-          "Initial setting of '",
-          deparse(substitute(init_list)),
-          "' in each chain is recommended to be differed."
-        ))
-      }
-    }
-  )
-}
-
 #' Processing Multiple Chain Record Result Matrix from `RcppEigen`
 #' 
 #' Preprocess multiple chain record matrix for [posterior::posterior] package.
@@ -124,9 +68,9 @@ isnot_identical <- function(init_list, case = c("dim", "values")) {
 split_paramarray <- function(x, chain, param_name) {
   num_var <- ncol(x) / chain
   res <- 
-    split.data.frame(t(x), gl(num_var, 1, ncol(x))) %>%
-    lapply(t) %>%
-    unlist() %>% 
+    split.data.frame(t(x), gl(num_var, 1, ncol(x))) |>
+    lapply(t) |>
+    unlist() |> 
     array(
       dim = c(nrow(x), chain, num_var),
       dimnames = list(
@@ -147,7 +91,7 @@ split_paramarray <- function(x, chain, param_name) {
 #' @noRd
 split_psirecord <- function(x, chain = 1, varname = "cholesky") {
   res <- 
-    x %>% 
+    x |> 
     split.data.frame(gl(nrow(x) / ncol(x), ncol(x)))
   if (chain == 1) {
     return(res)
@@ -156,9 +100,9 @@ split_psirecord <- function(x, chain = 1, varname = "cholesky") {
       res,
       function(y) {
         num_var <- ncol(y) / chain
-        split.data.frame(t(y), gl(num_var, 1, ncol(y))) %>% 
-          lapply(t) %>% 
-          unlist() %>% 
+        split.data.frame(t(y), gl(num_var, 1, ncol(y))) |> 
+          lapply(t) |> 
+          unlist() |> 
           array(
             dim = c(nrow(y), chain, num_var),
             dimnames = list(
@@ -188,10 +132,10 @@ split_chain <- function(x, chain = 1, varname = "alpha") {
     # num_var <- ncol(x) / chain
     num_row <- nrow(x) / chain
     res <-
-      # split.data.frame(t(x), gl(num_var, 1, ncol(x))) %>%
-      # lapply(t) %>%
-      split.data.frame(x, gl(chain, num_row)) %>%
-      unlist(x) %>%
+      # split.data.frame(t(x), gl(num_var, 1, ncol(x))) |>
+      # lapply(t) |>
+      split.data.frame(x, gl(chain, num_row)) |>
+      unlist(x) |>
       array(
         # dim = c(nrow(x), chain, num_var),
         dim = c(num_row, chain, ncol(x)),
@@ -227,6 +171,167 @@ get_gammaparam <- function(mode, sd) {
   )
 }
 
+#' Compute Summaries from Forecast Draws
+#' 
+#' @param draws Matrix in forms of rbind(step) x cbind(draws)
+#' @param n_ahead Forecast step used
+#' @param dim_data Dimension
+#' @param num_draw MCMC draws
+#' @param var_names Variable names
+#' @param level level for lower and upper quantiles
+#' @param med Get median instead of mean?
+#' @param roll Is the `draws` the result of rolling or expanding windows?
+#' 
+#' @noRd 
+process_forecast_draws <- function(draws, n_ahead, dim_data, num_draw, var_names, level = .05, roll = FALSE, med = FALSE) {
+  if (roll) {
+    if (med) {
+      pred_mean <-
+        draws |>
+        lapply(function(res) {
+          unlist(res) |>
+            array(dim = c(n_ahead, dim_data, num_draw)) |>
+            apply(c(1, 2), median)
+        })
+    } else {
+      pred_mean <-
+        draws |>
+        lapply(function(res) {
+          unlist(res) |>
+            array(dim = c(n_ahead, dim_data, num_draw)) |>
+            apply(c(1, 2), mean)
+        })
+    }
+    pred_mean <- do.call(rbind, pred_mean)
+    pred_se <-
+      draws |>
+      lapply(function(res) {
+        unlist(res) |>
+          array(dim = c(n_ahead, dim_data, num_draw)) |>
+          apply(c(1, 2), sd)
+      })
+    pred_se <- do.call(rbind, pred_se)
+    pred_lower <-
+      draws |> 
+      lapply(function(res) {
+        unlist(res) |> 
+          array(dim = c(n_ahead, dim_data, num_draw)) |> 
+          apply(c(1, 2), quantile, probs = level / 2)
+      })
+    pred_lower <- do.call(rbind, pred_lower)
+    pred_upper <-
+      draws |>
+      lapply(function(res) {
+        unlist(res) |>
+          array(dim = c(n_ahead, dim_data, num_draw)) |>
+          apply(c(1, 2), quantile, probs = 1 - level / 2)
+      })
+    pred_upper <- do.call(rbind, pred_upper)
+  } else {
+    mcmc_distn <-
+      draws |>
+      unlist() |>
+      array(dim = c(n_ahead, dim_data, num_draw))
+    if (med) {
+      pred_mean <- apply(mcmc_distn, c(1, 2), median)
+    } else {
+      pred_mean <- apply(mcmc_distn, c(1, 2), mean)
+    }
+    pred_se <- apply(mcmc_distn, c(1, 2), sd)
+    pred_lower <- apply(mcmc_distn, c(1, 2), quantile, probs = level / 2)
+    pred_upper <- apply(mcmc_distn, c(1, 2), quantile, probs = 1 - level / 2)
+  }
+  colnames(pred_mean) <- var_names
+  colnames(pred_se) <- var_names
+  colnames(pred_lower) <- var_names
+  colnames(pred_upper) <- var_names
+  if (nrow(pred_mean) == ncol(pred_mean)) {
+    rownames(pred_mean) <- var_names
+    rownames(pred_se) <- var_names
+    rownames(pred_lower) <- var_names
+    rownames(pred_upper) <- var_names
+  }
+  list(
+    mean = pred_mean,
+    sd = pred_se,
+    lower = pred_lower,
+    upper = pred_upper
+  )
+}
+
+#' Compute Summaries from Vector Draws
+#'
+#' @param dim_data Dimension
+#' @param level level for lower and upper quantiles
+#' @param med Get median instead of mean?
+#'
+#' @noRd
+process_vector_draws <- function(draws, dim_data, level = .05, med = FALSE) {
+  mcmc_distn <- matrix(draws, ncol = dim_data)
+  if (med) {
+    pred_mean <- apply(mcmc_distn, 2, median)
+  } else {
+    pred_mean <- colMeans(mcmc_distn)
+  }
+  list(
+    mean = pred_mean,
+    sd = apply(mcmc_distn, 2, sd),
+    lower = apply(mcmc_distn, 2, quantile, probs = level / 2),
+    upper = apply(mcmc_distn, 2, quantile, probs = 1 - level / 2)
+  )
+}
+
+#' Compute Summaries from Dynamic Spillover
+#'
+#' @param dim_data Dimension
+#' @param level level for lower and upper quantiles
+#' @param med Get median instead of mean?
+#' @param var_names Variable names
+#' 
+#' @importFrom tibble as_tibble
+#' @importFrom dplyr mutate
+#' @noRd 
+process_dynamic_spdraws <- function(draws, dim_data, level = .05, med = FALSE, var_names) {
+  sp_draws <- lapply(
+    draws,
+    function(x) {
+      do.call(
+        cbind,
+        process_vector_draws(unlist(x), dim_data = dim_data, level = level, med = med)
+      ) |>
+        as.data.frame() |>
+        mutate(series = var_names)
+    }
+  )
+  do.call(rbind, sp_draws) |> 
+    as_tibble()
+}
+
+#' Pivot longer spillover
+#' 
+#' @param connect Connectedness table
+#' @param col_names Column name for value
+#' @noRd 
+gather_spillover <- function(connect, col_names = "spillover") {
+  connect |>
+    as.data.frame() |>
+    rownames_to_column(var = "series") |>
+    pivot_longer(-"series", names_to = "shock", values_to = col_names)
+}
+
+#' Pivot longer spillover summaries
+#'
+#' @param distn Connectedness table distribution
+#' @param prefix Column names prefix
+#' 
+#' @noRd
+join_long_spillover <- function(connect, prefix = "spillover") {
+  gather_spillover(connect$mean, col_names = prefix) |>
+    left_join(gather_spillover(connect$lower, col_names = paste(prefix, "lower", sep = "_")), by = c("series", "shock")) |>
+    left_join(gather_spillover(connect$upper, col_names = paste(prefix, "upper", sep = "_")), by = c("series", "shock")) |>
+    left_join(gather_spillover(connect$sd, col_names = paste(prefix, "sd", sep = "_")), by = c("series", "shock"))
+}
+
 #' Define Minnesota Group Matrix
 #'
 #' This function creates a matrix with group index
@@ -284,4 +389,25 @@ build_grpmat <- function(p, dim_data, dim_design, num_coef, minnesota, include_m
       do.call(rbind, glob_idmat)
     }
   )
-}
\ No newline at end of file
+}
+
+#' Get MCMC records as a list
+#' 
+#' @param object Model list
+#' @param split_chain Split each chain as list
+#' @noRd
+get_records <- function(object, split_chain = TRUE) {
+  num_chains <- 1
+  if (split_chain) {
+    num_chains <- object$chain
+  }
+  lapply(
+    object$param_names,
+    function(x) {
+      subset_draws(object$param, variable = x) |>
+        as_draws_matrix() |>
+        split.data.frame(gl(num_chains, nrow(object$param) / num_chains))
+    }
+  ) |>
+    setNames(paste(object$param_names, "record", sep = "_"))
+}
