@@ -50,7 +50,7 @@ print.bvarflat <- function(x, digits = max(3L, getOption("digits") - 3L), ...) {
   cat("====================================================\n")
   cat("IW scale matrix:\n")
   print.default(
-    x$iw_scale,
+    x$covmat,
     digits = digits,
     print.gap = 2L,
     quote = FALSE
