@@ -39,9 +39,9 @@ spillover.olsmod <- function(object, n_ahead = 10L, ...) {
   colnames(res$connect) <- colnames(object$coefficients)
   rownames(res$connect) <- colnames(object$coefficients)
   res$df_long <-
-    res$connect %>%
-    as.data.frame() %>%
-    rownames_to_column(var = "series") %>%
+    res$connect |>
+    as.data.frame() |>
+    rownames_to_column(var = "series") |>
     pivot_longer(-"series", names_to = "shock", values_to = "spillover")
   colnames(res$net_pairwise) <- colnames(res$connect)
   rownames(res$net_pairwise) <- rownames(res$connect)
@@ -108,9 +108,9 @@ spillover.normaliw <- function(object, n_ahead = 10L, num_iter = 5000L, num_burn
   colnames(res$connect) <- colnames(object$coefficients)
   rownames(res$connect) <- colnames(object$coefficients)
   res$df_long <-
-    res$connect %>%
-    as.data.frame() %>%
-    rownames_to_column(var = "series") %>%
+    res$connect |>
+    as.data.frame() |>
+    rownames_to_column(var = "series") |>
     pivot_longer(-"series", names_to = "shock", values_to = "spillover")
   colnames(res$net_pairwise) <- colnames(res$connect)
   rownames(res$net_pairwise) <- rownames(res$connect)
@@ -123,71 +123,139 @@ spillover.normaliw <- function(object, n_ahead = 10L, num_iter = 5000L, num_burn
 }
 
 #' @rdname spillover
+#' @param level Specify alpha of confidence interval level 100(1 - alpha) percentage. By default, .05.
 #' @param sparse `r lifecycle::badge("experimental")` Apply restriction. By default, `FALSE`.
 #' @importFrom posterior subset_draws as_draws_matrix
+#' @importFrom dplyr left_join
 #' @export
-spillover.bvarldlt <- function(object, n_ahead = 10L, sparse = FALSE, ...) {
+spillover.bvarldlt <- function(object, n_ahead = 10L, level = .05, sparse = FALSE, ...) {
   alpha_record <- as_draws_matrix(subset_draws(object$param, variable = "alpha"))
-  a_record <- as_draws_matrix(subset_draws(object$param, variable = "a"))
-  if (sparse) {
-    alpha_record <- as_draws_matrix(subset_draws(object$param, variable = "alpha_sparse"))
-    a_record <- as_draws_matrix(subset_draws(object$param, variable = "a_sparse"))
-  }
-  res <- compute_varldlt_spillover(
+  # a_record <- as_draws_matrix(subset_draws(object$param, variable = "a"))
+  # if (sparse) {
+  #   alpha_record <- as_draws_matrix(subset_draws(object$param, variable = "alpha_sparse"))
+  #   a_record <- as_draws_matrix(subset_draws(object$param, variable = "a_sparse"))
+  # }
+  fit_ls <- get_records(object, FALSE)
+  sp_res <- compute_varldlt_spillover(
     object$p,
     step = n_ahead,
-    alpha_record = alpha_record,
-    d_record = as_draws_matrix(subset_draws(object$param, variable = "d")),
-    a_record = a_record
+    # alpha_record = alpha_record,
+    # d_record = as_draws_matrix(subset_draws(object$param, variable = "d")),
+    # a_record = a_record
+    fit_record = fit_ls,
+    sparse = sparse
+  )
+  dim_data <- object$m
+  num_draw <- nrow(alpha_record)
+  var_names <- colnames(object$coefficients)
+  connect_distn <- process_forecast_draws(
+    sp_res$connect,
+    n_ahead = dim_data,
+    dim_data = dim_data,
+    num_draw = num_draw,
+    var_names = var_names,
+    level = level,
+    med = FALSE
+  )
+  net_pairwise_distn <- process_forecast_draws(
+    sp_res$net_pairwise,
+    n_ahead = dim_data,
+    dim_data = dim_data,
+    num_draw = num_draw,
+    var_names = var_names,
+    level = level,
+    med = FALSE
+  )
+  tot_distn <- process_vector_draws(sp_res$tot, dim_data = 1, level = level, med = FALSE)
+  to_distn <- process_vector_draws(sp_res$to, dim_data = dim_data, level = level, med = FALSE)
+  from_distn <- process_vector_draws(sp_res$from, dim_data = dim_data, level = level, med = FALSE)
+  net_distn <- process_vector_draws(sp_res$net, dim_data = dim_data, level = level, med = FALSE)
+  df_long <-
+    join_long_spillover(connect_distn, prefix = "spillover") |>
+    left_join(join_long_spillover(net_pairwise_distn, prefix = "net"), by = c("series", "shock"))
+  res <- list(
+    connect = connect_distn,
+    net_pairwise = net_pairwise_distn,
+    tot = tot_distn,
+    to = to_distn,
+    from = from_distn,
+    net = net_distn,
+    df_long = df_long,
+    ahead = n_ahead,
+    process = object$process
   )
-  colnames(res$connect) <- colnames(object$coefficients)
-  rownames(res$connect) <- colnames(object$coefficients)
-  res$df_long <-
-    res$connect %>%
-    as.data.frame() %>%
-    rownames_to_column(var = "series") %>%
-    pivot_longer(-"series", names_to = "shock", values_to = "spillover")
-  colnames(res$net_pairwise) <- colnames(res$connect)
-  rownames(res$net_pairwise) <- rownames(res$connect)
-  res$connect <- rbind(res$connect, "to_spillovers" = res$to)
-  res$connect <- cbind(res$connect, "from_spillovers" = c(res$from, res$tot))
-  res$ahead <- n_ahead
-  res$process <- object$process
   class(res) <- "bvharspillover"
   res
 }
 
 #' @rdname spillover
+#' @param level Specify alpha of confidence interval level 100(1 - alpha) percentage. By default, .05.
 #' @param sparse `r lifecycle::badge("experimental")` Apply restriction. By default, `FALSE`.
 #' @importFrom posterior subset_draws as_draws_matrix
+#' @importFrom dplyr left_join
 #' @export
-spillover.bvharldlt <- function(object, n_ahead = 10L, sparse = FALSE, ...) {
+spillover.bvharldlt <- function(object, n_ahead = 10L, level = .05, sparse = FALSE, ...) {
   phi_record <- as_draws_matrix(subset_draws(object$param, variable = "phi"))
-  a_record <- as_draws_matrix(subset_draws(object$param, variable = "a"))
-  if (sparse) {
-    phi_record <- as_draws_matrix(subset_draws(object$param, variable = "phi_sparse"))
-    a_record <- as_draws_matrix(subset_draws(object$param, variable = "a_sparse"))
-  }
-  res <- compute_vharldlt_spillover(
+  # a_record <- as_draws_matrix(subset_draws(object$param, variable = "a"))
+  # if (sparse) {
+  #   phi_record <- as_draws_matrix(subset_draws(object$param, variable = "phi_sparse"))
+  #   a_record <- as_draws_matrix(subset_draws(object$param, variable = "a_sparse"))
+  # }
+  fit_ls <- get_records(object, FALSE)
+  sp_res <- compute_vharldlt_spillover(
     object$week, object$month,
     step = n_ahead,
-    phi_record = phi_record,
-    d_record = as_draws_matrix(subset_draws(object$param, variable = "d")),
-    a_record = a_record
+    # phi_record = phi_record,
+    # d_record = as_draws_matrix(subset_draws(object$param, variable = "d")),
+    # a_record = a_record
+    fit_record = fit_ls,
+    sparse = sparse
   )
-  colnames(res$connect) <- colnames(object$coefficients)
-  rownames(res$connect) <- colnames(object$coefficients)
-  res$df_long <-
-    res$connect %>%
-    as.data.frame() %>%
-    rownames_to_column(var = "series") %>%
-    pivot_longer(-"series", names_to = "shock", values_to = "spillover")
-  colnames(res$net_pairwise) <- colnames(res$connect)
-  rownames(res$net_pairwise) <- rownames(res$connect)
-  res$connect <- rbind(res$connect, "to_spillovers" = res$to)
-  res$connect <- cbind(res$connect, "from_spillovers" = c(res$from, res$tot))
-  res$ahead <- n_ahead
-  res$process <- object$process
+  dim_data <- object$m
+  num_draw <- nrow(phi_record)
+  var_names <- colnames(object$coefficients)
+  connect_distn <- process_forecast_draws(
+    sp_res$connect,
+    n_ahead = dim_data,
+    dim_data = dim_data,
+    num_draw = num_draw,
+    var_names = var_names,
+    level = level,
+    med = FALSE
+  )
+  net_pairwise_distn <- process_forecast_draws(
+    sp_res$net_pairwise,
+    n_ahead = dim_data,
+    dim_data = dim_data,
+    num_draw = num_draw,
+    var_names = var_names,
+    level = level,
+    med = FALSE
+  )
+  tot_distn <- process_vector_draws(sp_res$tot, dim_data = 1, level = level, med = FALSE)
+  to_distn <- process_vector_draws(sp_res$to, dim_data = dim_data, level = level, med = FALSE)
+  from_distn <- process_vector_draws(sp_res$from, dim_data = dim_data, level = level, med = FALSE)
+  net_distn <- process_vector_draws(sp_res$net, dim_data = dim_data, level = level, med = FALSE)
+  df_long <-
+    join_long_spillover(connect_distn, prefix = "spillover") |>
+    left_join(join_long_spillover(net_pairwise_distn, prefix = "net"), by = c("series", "shock"))
+  res <- list(
+    connect = connect_distn,
+    net_pairwise = net_pairwise_distn,
+    tot = tot_distn,
+    to = to_distn,
+    from = from_distn,
+    net = net_distn,
+    df_long = df_long,
+    ahead = n_ahead,
+    process = object$process
+  )
+  # colnames(res$net_pairwise) <- colnames(res$connect)
+  # rownames(res$net_pairwise) <- rownames(res$connect)
+  # res$connect <- rbind(res$connect, "to_spillovers" = res$to)
+  # res$connect <- cbind(res$connect, "from_spillovers" = c(res$from, res$tot))
+  # res$ahead <- n_ahead
+  # res$process <- object$process
   class(res) <- "bvharspillover"
   res
 }
@@ -315,7 +383,7 @@ dynamic_spillover.normaliw <- function(object, n_ahead = 10L, window,
         # num_chains = num_chains, num_iter = object$iter, num_burn = object$burn, thin = object$thin,
         lag = object$p, bayes_spec = object$spec, include_mean = include_mean,
         seed_chain = sample.int(.Machine$integer.max, size = num_horizon),
-        # seed_chain = sample.int(.Machine$integer.max, size = num_chains * num_horizon) %>% matrix(ncol = num_chains),
+        # seed_chain = sample.int(.Machine$integer.max, size = num_chains * num_horizon) |> matrix(ncol = num_chains),
         nthreads = num_thread
       )
     },
@@ -326,7 +394,7 @@ dynamic_spillover.normaliw <- function(object, n_ahead = 10L, window,
         # num_chains = num_chains, num_iter = object$iter, num_burn = object$burn, thin = object$thin,
         week = object$week, month = object$month, bayes_spec = object$spec, include_mean = include_mean,
         seed_chain = sample.int(.Machine$integer.max, size = num_horizon),
-        # seed_chain = sample.int(.Machine$integer.max, size = num_chains * num_horizon) %>% matrix(ncol = num_chains),
+        # seed_chain = sample.int(.Machine$integer.max, size = num_chains * num_horizon) |> matrix(ncol = num_chains),
         nthreads = num_thread
       )
     },
@@ -353,10 +421,12 @@ dynamic_spillover.normaliw <- function(object, n_ahead = 10L, window,
 
 #' @rdname dynamic_spillover
 #' @param window Window size
+#' @param level Specify alpha of confidence interval level 100(1 - alpha) percentage. By default, .05.
 #' @param sparse `r lifecycle::badge("experimental")` Apply restriction. By default, `FALSE`.
 #' @param num_thread `r lifecycle::badge("experimental")` Number of threads
+#' @importFrom dplyr mutate
 #' @export
-dynamic_spillover.ldltmod <- function(object, n_ahead = 10L, window, sparse = FALSE, num_thread = 1, ...) {
+dynamic_spillover.ldltmod <- function(object, n_ahead = 10L, window, level = .05, sparse = FALSE, num_thread = 1, ...) {
   num_horizon <- nrow(object$y) - window + 1
   if (num_horizon < 0) {
     stop(sprintf("Invalid 'window' size: Specify as 'window' < 'nrow(y) + 1' = %d", nrow(object$y) + 1))
@@ -384,6 +454,7 @@ dynamic_spillover.ldltmod <- function(object, n_ahead = 10L, window, sparse = FA
     if (object$spec$hierarchical) {
       param_prior$shape <- object$spec$lambda$param[1]
       param_prior$rate <- object$spec$lambda$param[2]
+      param_prior$grid_size <- object$spec$lambda$grid_size
       prior_nm <- "MN_Hierarchical"
     }
   } else if (prior_nm == "SSVS") {
@@ -401,20 +472,28 @@ dynamic_spillover.ldltmod <- function(object, n_ahead = 10L, window, sparse = FA
     "Horseshoe" = 3,
     "MN_Hierarchical" = 4,
     "NG" = 5,
-    "DL" = 6
+    "DL" = 6,
+    "GDP" = 7
   )
   grp_id <- unique(c(object$group))
-  if (length(grp_id) > 1) {
-    own_id <- 2
-    cross_id <- seq_len(object$p + 1)[-2]
-  } else {
-    own_id <- 1
-    cross_id <- 2
-  }
+  # if (length(grp_id) > 1) {
+  #   own_id <- 2
+  #   cross_id <- seq_len(object$p + 1)[-2]
+  # } else {
+  #   own_id <- 1
+  #   cross_id <- 2
+  # }
   num_chains <- object$chain
-  chunk_size <- num_horizon * num_chains %/% num_thread # default setting of OpenMP schedule(static)
+  # chunk_size <- num_horizon * num_chains %/% num_thread # default setting of OpenMP schedule(static)
   sp_list <- switch(model_type,
     "bvarldlt" = {
+      if (length(grp_id) > 1) {
+        own_id <- 2
+        cross_id <- seq_len(object$p + 1)[-2]
+      } else {
+        own_id <- 1
+        cross_id <- 2
+      }
       dynamic_bvarldlt_spillover(
         y = object$y, window = window, step = n_ahead,
         num_chains = num_chains,
@@ -426,51 +505,76 @@ dynamic_spillover.ldltmod <- function(object, n_ahead = 10L, window, sparse = FA
         # param_init = object$init[[1]], # should add multiple chain later
         param_init = object$init,
         prior_type = prior_type,
+        ggl = object$ggl,
         grp_id = grp_id, own_id = own_id, cross_id = cross_id, grp_mat = object$group,
         include_mean = include_mean,
         # seed_chain = sample.int(.Machine$integer.max, size = num_horizon),
-        seed_chain = sample.int(.Machine$integer.max, size = num_chains * num_horizon) %>% matrix(ncol = num_chains),
-        nthreads = num_thread,
-        chunk_size = chunk_size
+        seed_chain = sample.int(.Machine$integer.max, size = num_chains * num_horizon) |> matrix(ncol = num_chains),
+        nthreads = num_thread
       )
     },
     "bvharldlt" = {
+      if (length(grp_id) > 1) {
+        own_id <- c(2, 4, 6)
+        cross_id <- c(1, 3, 5)
+      } else {
+        own_id <- 1
+        cross_id <- 2
+      }
       dynamic_bvharldlt_spillover(
         y = object$y, window = window, step = n_ahead,
         num_chains = num_chains,
         num_iter = object$iter, num_burn = object$burn, thin = object$thin, sparse = sparse,
-        week = object$p, month = object$month,
+        week = object$week, month = object$month,
         param_reg = object$sv[c("shape", "scale")],
         param_prior = param_prior,
         param_intercept = object$intercept[c("mean_non", "sd_non")],
         param_init = object$init,
         prior_type = prior_type,
+        ggl = object$ggl,
         grp_id = grp_id, own_id = own_id, cross_id = cross_id, grp_mat = object$group,
         include_mean = include_mean,
-        seed_chain = sample.int(.Machine$integer.max, size = num_chains * num_horizon) %>% matrix(ncol = num_chains),
-        nthreads = num_thread,
-        chunk_size = chunk_size
+        seed_chain = sample.int(.Machine$integer.max, size = num_chains * num_horizon) |> matrix(ncol = num_chains),
+        nthreads = num_thread
       )
     },
     stop("Not supported model.")
   )
-  sp_list <- lapply(sp_list, function(x) {
-    if (is.matrix(x)) {
-      return(apply(x, 1, mean))
-    }
-    Reduce("+", x) / length(x)
-  })
+  dim_data <- object$m
+  var_names <- colnames(object$coefficients)
+  to_distn <- process_dynamic_spdraws(sp_list$to, dim_data = dim_data, level = level, med = FALSE, var_names = var_names)
+  from_distn <- process_dynamic_spdraws(sp_list$from, dim_data = dim_data, level = level, med = FALSE, var_names = var_names)
+  net_distn <- process_dynamic_spdraws(sp_list$net, dim_data = dim_data, level = level, med = FALSE, var_names = var_names)
+  tot_distn <-
+    lapply(
+      sp_list$tot,
+      function(x) {
+        processed <- process_vector_draws(unlist(x), dim_data = 1, level = level, med = FALSE)
+        do.call(cbind, processed)
+      }
+    )
+  tot_distn <- do.call(rbind, tot_distn)
+  # sp_list <- lapply(sp_list, function(x) {
+  #   if (is.matrix(x)) {
+  #     return(apply(x, 1, mean))
+  #   }
+  #   Reduce("+", x) / length(x)
+  # })
   # colnames(sp_list$to) <- paste(colnames(object$y), "to", sep = "_")
   # colnames(sp_list$from) <- paste(colnames(object$y), "from", sep = "_")
-  colnames(sp_list$to) <- colnames(object$y)
-  colnames(sp_list$from) <- colnames(object$y)
-  colnames(sp_list$net) <- colnames(object$y)
+  # colnames(sp_list$to) <- colnames(object$y)
+  # colnames(sp_list$from) <- colnames(object$y)
+  # colnames(sp_list$net) <- colnames(object$y)
   res <- list(
-    tot = sp_list$tot,
+    # tot = sp_list$tot,
+    tot = tot_distn,
     # directional = as_tibble(cbind(sp_list$to, sp_list$from)),
-    to = as_tibble(sp_list$to),
-    from = as_tibble(sp_list$from),
-    net = as_tibble(sp_list$net),
+    # to = as_tibble(sp_list$to),
+    # from = as_tibble(sp_list$from),
+    # net = as_tibble(sp_list$net),
+    to = to_distn,
+    from = from_distn,
+    net = net_distn,
     index = window:nrow(object$y),
     ahead = n_ahead,
     process = object$process
@@ -480,11 +584,12 @@ dynamic_spillover.ldltmod <- function(object, n_ahead = 10L, window, sparse = FA
 }
 
 #' @rdname dynamic_spillover
+#' @param level Specify alpha of confidence interval level 100(1 - alpha) percentage. By default, .05.
 #' @param sparse `r lifecycle::badge("experimental")` Apply restriction. By default, `FALSE`.
 #' @param num_thread `r lifecycle::badge("experimental")` Number of threads
 #' @importFrom posterior subset_draws as_draws_matrix
 #' @export
-dynamic_spillover.svmod <- function(object, n_ahead = 10L, sparse = FALSE, num_thread = 1, ...) {
+dynamic_spillover.svmod <- function(object, n_ahead = 10L, level = .05, sparse = FALSE, num_thread = 1, ...) {
   num_design <- nrow(object$y0)
   if (num_design < 0) {
     stop(sprintf("Invalid 'window' size: Specify as 'window' < 'nrow(y) + 1' = %d", nrow(object$y) + 1))
@@ -500,50 +605,53 @@ dynamic_spillover.svmod <- function(object, n_ahead = 10L, sparse = FALSE, num_t
   }
   model_type <- class(object)[1]
   include_mean <- ifelse(object$type == "const", TRUE, FALSE)
+  fit_ls <- get_records(object, FALSE)
   sp_list <- switch(model_type,
     "bvarsv" = {
-      alpha_record <- as_draws_matrix(subset_draws(object$param, variable = "alpha"))
-      a_record <- as_draws_matrix(subset_draws(object$param, variable = "a"))
-      if (sparse) {
-        alpha_record <- as_draws_matrix(subset_draws(object$param, variable = "alpha_sparse"))
-        a_record <- as_draws_matrix(subset_draws(object$param, variable = "a_sparse"))
-      }
       dynamic_bvarsv_spillover(
         lag = object$p, step = n_ahead, num_design = num_design,
-        alpha_record = alpha_record,
-        h_record = as_draws_matrix(subset_draws(object$param, variable = "h")),
-        a_record = a_record,
+        fit_record = fit_ls, sparse = sparse, include_mean = include_mean,
         nthreads = num_thread
       )
     },
     "bvharsv" = {
-      phi_record <- as_draws_matrix(subset_draws(object$param, variable = "phi"))
-      a_record <- as_draws_matrix(subset_draws(object$param, variable = "a"))
-      if (sparse) {
-        phi_record <- as_draws_matrix(subset_draws(object$param, variable = "phi_sparse"))
-        a_record <- as_draws_matrix(subset_draws(object$param, variable = "a_sparse"))
-      }
       dynamic_bvharsv_spillover(
         week = object$week, month = object$month, step = n_ahead, num_design = num_design,
-        phi_record = phi_record,
-        h_record = as_draws_matrix(subset_draws(object$param, variable = "h")),
-        a_record = a_record,
+        fit_record = fit_ls, sparse = sparse, include_mean = include_mean,
         nthreads = num_thread
       )
     },
     stop("Not supported model.")
   )
+  dim_data <- object$m
+  var_names <- colnames(object$coefficients)
+  to_distn <- process_dynamic_spdraws(sp_list$to, dim_data = dim_data, level = level, med = FALSE, var_names = var_names)
+  from_distn <- process_dynamic_spdraws(sp_list$from, dim_data = dim_data, level = level, med = FALSE, var_names = var_names)
+  net_distn <- process_dynamic_spdraws(sp_list$net, dim_data = dim_data, level = level, med = FALSE, var_names = var_names)
+  tot_distn <-
+    lapply(
+      sp_list$tot,
+      function(x) {
+        processed <- process_vector_draws(unlist(x), dim_data = 1, level = level, med = FALSE)
+        do.call(cbind, processed)
+      }
+    )
+  tot_distn <- do.call(rbind, tot_distn)
   # colnames(sp_list$to) <- paste(colnames(object$y), "to", sep = "_")
   # colnames(sp_list$from) <- paste(colnames(object$y), "from", sep = "_")
-  colnames(sp_list$to) <- colnames(object$y)
-  colnames(sp_list$from) <- colnames(object$y)
-  colnames(sp_list$net) <- colnames(object$y)
+  # colnames(sp_list$to) <- colnames(object$y)
+  # colnames(sp_list$from) <- colnames(object$y)
+  # colnames(sp_list$net) <- colnames(object$y)
   res <- list(
-    tot = sp_list$tot,
+    # tot = sp_list$tot,
+    tot = tot_distn,
     # directional = as_tibble(cbind(sp_list$to, sp_list$from)),
-    to = as_tibble(sp_list$to),
-    from = as_tibble(sp_list$from),
-    net = as_tibble(sp_list$net),
+    # to = as_tibble(sp_list$to),
+    # from = as_tibble(sp_list$from),
+    # net = as_tibble(sp_list$net),
+    to = to_distn,
+    from = from_distn,
+    net = net_distn,
     index = seq_len(nrow(object$y))[-seq_len(nrow(object$y) - nrow(object$y0))],
     ahead = n_ahead,
     process = object$process
