@@ -575,119 +575,3 @@ choose_bayes <- function(bayes_bound = bound_bvhar(),
     }
   )
 }
-
-#' Choose the Hyperparameters Set of SSVS-VAR using a Default Semiautomatic Approach
-#' 
-#' `r lifecycle::badge("deprecated")`
-#' This function chooses \eqn{(\tau_{0i}, \tau_{1i})} and \eqn{(\kappa_{0i}, \kappa_{1i})}
-#' using a default semiautomatic approach.
-#' 
-#' @param y Time series data of which columns indicate the variables.
-#' @param ord Order for VAR or VHAR.
-#' @param type Model type (Default: `VAR` or `VHAR`).
-#' @param param Preselected constants \eqn{c_0 << c_1}. By default, `0.1` and `10` (See Details).
-#' @param include_mean Add constant term (Default: `TRUE`) or not (`FALSE`).
-#' @param gamma_param Parameters (shape, rate) for Gamma distribution. This is for the output.
-#' @param mean_non Prior mean of unrestricted coefficients. This is for the output.
-#' @param sd_non Standard deviance of unrestricted coefficients. This is for the output.
-#' @details 
-#' Instead of using subjective values of \eqn{(\tau_{0i}, \tau_{1i})}, we can use
-#' \deqn{\tau_{ki} = c_k \hat{VAR(OLS)}}
-#' It must be \eqn{c_0 << c_1}.
-#' 
-#' In case of \eqn{(\omega_{0ij}, \omega_{1ij})},
-#' \deqn{\omega_{kij} = c_k = \hat{VAR(OLS)}}
-#' similarly.
-#' @return `ssvsinput` object
-#' @references 
-#' George, E. I., & McCulloch, R. E. (1993). *Variable Selection via Gibbs Sampling*. Journal of the American Statistical Association, 88(423), 881-889.
-#' 
-#' George, E. I., Sun, D., & Ni, S. (2008). *Bayesian stochastic search for VAR model restrictions*. Journal of Econometrics, 142(1), 553-580.
-#' 
-#' Koop, G., & Korobilis, D. (2009). *Bayesian Multivariate Time Series Methods for Empirical Macroeconomics*. Foundations and Trends® in Econometrics, 3(4), 267-358.
-#' @export
-choose_ssvs <- function(y, 
-                        ord, 
-                        type = c("VAR", "VHAR"), 
-                        param = c(.1, 10),
-                        include_mean = TRUE,
-                        gamma_param = c(.01, .01),
-                        mean_non = 0,
-                        sd_non = .1) {
-  deprecate_warn("2.0.1", "choose_ssvs()", details = "'var_bayes()' and 'vhar_bayes()' do full bayesian approaches instead of semi-automatic.")
-  type <- match.arg(type)
-  if (param[1] >= param[2]) {
-    stop("'param[2]' should be larger than 'param[1]'.")
-  }
-  res <- switch(
-    type,
-    "VAR" = {
-      fit <- var_lm(y, p = ord, include_mean = FALSE)
-      fit_infer <- infer_var(fit)$summary_stat
-      mean_coef <- fit_infer[,1]
-      sd_coef <- fit_infer[,2]
-      if (include_mean) {
-        fit <- var_lm(y, p = ord, include_mean = TRUE)
-        mean_non <- fit$coef[fit$df,]
-      }
-      sd_chol <- chol(fit$covmat)
-      sd_chol <- sd_chol[upper.tri(sd_chol, diag = FALSE)]
-      list(
-        coef_spike = param[1] * sd_coef,
-        coef_slab = param[2] * sd_coef,
-        coef_mixture = .5,
-        coef_s1 = c(1, 1),
-        coef_s2 = c(1, 1),
-        # mean_coef = mean_coef,
-        mean_non = mean_non,
-        sd_non = sd_non,
-        process = "VAR",
-        prior = "SSVS",
-        shape = gamma_param[1],
-        rate = gamma_param[2],
-        chol_spike = param[1] * sd_chol,
-        chol_slab = param[2] * sd_chol,
-        chol_mixture = .5,
-        chol_s1 = 1,
-        chol_s2 = 1
-      )
-    },
-    "VHAR" = {
-      if (missing(ord)) {
-        ord <- c(5, 22)
-      }
-      fit <- vhar_lm(y, har = ord, include_mean = include_mean)
-      fit_infer <- infer_vhar(fit)$summary_stat
-      sd_coef <- fit_infer[,2]
-      
-      if (include_mean) {
-        id_const <- seq(from = fit$df, to = fit$df * fit$m, by = fit$df)
-        mean_non <- fit_infer[id_const, 1]
-        sd_coef <- fit_infer[-id_const]
-      }
-      sd_chol <- chol(fit$covmat)
-      sd_chol <- sd_chol[upper.tri(sd_chol, diag = FALSE)]
-      list(
-        coef_spike = param[1] * sd_coef,
-        coef_slab = param[2] * sd_coef,
-        coef_mixture = .5,
-        coef_s1 = c(1, 1),
-        coef_s2 = c(1, 1),
-        # mean_coef = mean_coef,
-        mean_non = mean_non,
-        sd_non = sd_non,
-        process = "VHAR",
-        prior = "SSVS",
-        shape = gamma_param[1],
-        rate = gamma_param[2],
-        chol_spike = param[1] * sd_chol,
-        chol_slab = param[2] * sd_chol,
-        chol_mixture = .5,
-        chol_s1 = 1,
-        chol_s2 = 1
-      )
-    }
-  )
-  class(res) <- "ssvsinput"
-  res
-}
